
(*
    This file is part of DynLoader.

    DynLoader - cross-platform dynamic module loader.
    Author: Romiras (Feb, 2010)
    License: GPL v2 license
*)

(* Win32 implementation of DynLoader *)

{$IFDEF FPC}
    {$mode delphi}
{$ENDIF}

Unit HostDynLoader;

Interface

Uses DynLoader, Windows; 

Type
    // Handle to a loaded DLL
    TModuleHandle = HINST;

    TDynLoadModule = class (TDynLoadModuleBase)
        ModuleHandle: TModuleHandle;
        function LoadModule (FileName: TModuleName): Boolean; override;
        function LoadModuleEx (FileName: TModuleName; Flags: Cardinal): Boolean; override;
        procedure UnloadModule; override;
        function GetModuleSymbol (SymbolName: TSymbolName): Pointer; override;
        function GetModuleSymbolEx (SymbolName: TSymbolName; var Accu: Boolean): Pointer; override;
    end;

Const
    // Value designating an unassigned TModuleHandle of a failed loading
    INVALID_MODULEHANDLE_VALUE = TModuleHandle (Nil);

Implementation

function TDynLoadModule.LoadModule (FileName: TModuleName): Boolean;
begin
    if ModuleHandle = INVALID_MODULEHANDLE_VALUE then
        ModuleHandle := LoadLibrary (FileName);
    Result := ModuleHandle <> INVALID_MODULEHANDLE_VALUE
end;

function TDynLoadModule.LoadModuleEx (FileName: TModuleName; Flags: Cardinal): Boolean;
begin
    if ModuleHandle = INVALID_MODULEHANDLE_VALUE then
        ModuleHandle := LoadLibraryEx ( FileName, 0, Flags );
    Result := ModuleHandle <> INVALID_MODULEHANDLE_VALUE;
end;

procedure TDynLoadModule.UnloadModule;
begin
    if ModuleHandle <> INVALID_MODULEHANDLE_VALUE then
        FreeLibrary (ModuleHandle);
    ModuleHandle := INVALID_MODULEHANDLE_VALUE
end;

function TDynLoadModule.GetModuleSymbol (SymbolName: TSymbolName): Pointer;
begin
    if ModuleHandle <> INVALID_MODULEHANDLE_VALUE then
        Result := GetProcAddress (ModuleHandle, SymbolName)
    else
        Result := Nil
end;

function TDynLoadModule.GetModuleSymbolEx (SymbolName: TSymbolName; var Accu: Boolean): Pointer;
begin
    if ModuleHandle <> INVALID_MODULEHANDLE_VALUE then
        Result := GetProcAddress (ModuleHandle, SymbolName)
    else
        Result := Nil;
    Accu := Accu And (Result <> Nil)
end;

end.
